@@ -73,6 +73,11 @@ static void redraw_custom_statusline(win_T *wp);
 static int  did_update_one_window;
 #endif
 
+#ifdef FEAT_EVAL
+static void redraw_listener_cleanup(void);
+static void invoke_redraw_listener_start_or_end(bool start);
+#endif
+
 /*
  * Based on the current value of curwin->w_topline, transfer a screenfull
  * of stuff from Filemem to ScreenLines[], and update curwin->w_botline.
@@ -209,6 +214,9 @@ update_screen(int type_arg)
 		    redraw_cmdline = TRUE;
 		redraw_tabline = TRUE;
 	    }
+#if defined(FEAT_TABPANEL)
+	    redraw_tabpanel = TRUE;
+#endif
 	}
 	msg_scrolled = 0;
 	need_wait_return = FALSE;
@@ -241,6 +249,10 @@ update_screen(int type_arg)
 	curwin->w_redr_type = UPD_NOT_VALID;
 #endif
 
+#ifdef FEAT_EVAL
+    invoke_redraw_listener_start_or_end(true);
+#endif
+
     // Only start redrawing if there is really something to do.
     if (type == UPD_INVERTED)
 	update_curswant();
@@ -265,6 +277,11 @@ update_screen(int type_arg)
     if (redraw_tabline || type >= UPD_NOT_VALID)
 	draw_tabline();
 
+#if defined(FEAT_TABPANEL)
+    if (redraw_tabpanel || type >= UPD_NOT_VALID)
+	draw_tabpanel();
+#endif
+
 #ifdef FEAT_SYN_HL
     // Correct stored syntax highlighting info for changes in each displayed
     // buffer.  Each buffer must only be done once.
@@ -397,6 +414,12 @@ update_screen(int type_arg)
 	gui_update_scrollbars(FALSE);
     }
 #endif
+
+#ifdef FEAT_EVAL
+    invoke_redraw_listener_start_or_end(false);
+    redraw_listener_cleanup();
+#endif
+
     return OK;
 }
 
@@ -494,7 +517,7 @@ win_redr_status(win_T *wp, int ignore_pum UNUSED)
 	if (wp->w_buffer->b_p_ro)
 	    plen += vim_snprintf((char *)p + plen, MAXPATHL - plen, "%s", _("[RO]"));
 
-	this_ru_col = ru_col - (Columns - wp->w_width);
+	this_ru_col = ru_col - (cmdline_width - wp->w_width);
 	n = (wp->w_width + 1) / 2;
 	if (this_ru_col < n)
 	    this_ru_col = n;
@@ -532,7 +555,6 @@ win_redr_status(win_T *wp, int ignore_pum UNUSED)
 	screen_puts(p, row, wp->w_wincol, attr);
 	screen_fill(row, row + 1, plen + wp->w_wincol,
 			this_ru_col + wp->w_wincol, fillchar, fillchar, attr);
-
 	if ((NameBufflen = get_keymap_str(wp, (char_u *)"<%s>", NameBuff, MAXPATHL)) > 0
 		&& (this_ru_col - plen) > (NameBufflen + 1))
 	    screen_puts(NameBuff, row, (int)(this_ru_col - NameBufflen
@@ -620,6 +642,11 @@ showruler(int always)
     // Redraw the tab pages line if needed.
     if (redraw_tabline)
 	draw_tabline();
+
+#if defined(FEAT_TABPANEL)
+    if (redraw_tabpanel)
+	draw_tabpanel();
+#endif
 }
 
     void
@@ -709,8 +736,8 @@ win_redr_ruler(win_T *wp, int always, int ignore_pum)
 	    row = Rows - 1;
 	    fillchar = ' ';
 	    attr = 0;
-	    width = Columns;
-	    off = 0;
+	    off = cmdline_col_off;
+	    width = cmdline_width;
 	}
 
 	// In list mode virtcol needs to be recomputed
@@ -722,7 +749,10 @@ win_redr_ruler(win_T *wp, int always, int ignore_pum)
 	    wp->w_p_list = TRUE;
 	}
 
-	bufferlen = vim_snprintf((char *)buffer, RULER_BUF_LEN, "%ld,",
+	// row number, column number is appended
+	// l10n: leave as-is unless a space after the comma is preferred
+	// l10n: do not add any row/column label, due to the limited space
+	bufferlen = vim_snprintf((char *)buffer, RULER_BUF_LEN, _("%ld,"),
 		(wp->w_buffer->b_ml.ml_flags & ML_EMPTY)
 		    ? 0L
 		    : (long)(wp->w_cursor.lnum));
@@ -740,7 +770,7 @@ win_redr_ruler(win_T *wp, int always, int ignore_pum)
 	if (wp->w_status_height == 0)	// can't use last char of screen
 	    ++n1;
 
-	this_ru_col = ru_col - (Columns - width);
+	this_ru_col = ru_col - (cmdline_width - width);
 	// Never use more than half the window/screen width, leave the other
 	// half for the filename.
 	n2 = (width + 1) / 2;
@@ -868,12 +898,12 @@ text_to_screenline(win_T *wp, char_u *text, int col)
 	{
 	    cells = (*mb_ptr2cells)(p);
 	    c_len = (*mb_ptr2len)(p);
-	    if (col + cells > wp->w_width
-# ifdef FEAT_RIGHTLEFT
-		    - (wp->w_p_rl ? col : 0)
-# endif
-		    )
+	    if (col + cells > wp->w_width)
 		break;
+# ifdef FEAT_RIGHTLEFT
+	    if (wp->w_p_rl)
+		idx = off + wp->w_width - col - cells;
+# endif
 	    ScreenLines[idx] = *p;
 	    if (enc_utf8)
 	    {
@@ -881,13 +911,13 @@ text_to_screenline(win_T *wp, char_u *text, int col)
 		if (*p < 0x80 && u8cc[0] == 0)
 		{
 		    ScreenLinesUC[idx] = 0;
-#ifdef FEAT_ARABIC
+# ifdef FEAT_ARABIC
 		    prev_c = u8c;
-#endif
+# endif
 		}
 		else
 		{
-#ifdef FEAT_ARABIC
+# ifdef FEAT_ARABIC
 		    if (p_arshape && !p_tbidi && ARABIC_CHAR(u8c))
 		    {
 			// Do Arabic shaping.
@@ -918,7 +948,7 @@ text_to_screenline(win_T *wp, char_u *text, int col)
 		    }
 		    else
 			prev_c = u8c;
-#endif
+# endif
 		    // Non-BMP character: display as ? or fullwidth ?.
 		    ScreenLinesUC[idx] = u8c;
 		    for (i = 0; i < Screen_mco; ++i)
@@ -951,11 +981,11 @@ text_to_screenline(win_T *wp, char_u *text, int col)
 	    len = n;
 	if (len > 0)
 	{
-#ifdef FEAT_RIGHTLEFT
+# ifdef FEAT_RIGHTLEFT
 	    if (wp->w_p_rl)
 		mch_memmove(current_ScreenLine, text, len);
 	    else
-#endif
+# endif
 		mch_memmove(current_ScreenLine + col, text, len);
 	    col += len;
 	}
@@ -1026,12 +1056,12 @@ redraw_win_toolbar(win_T *wp)
     }
     wp->w_winbar_items[item_idx].wb_menu = NULL; // end marker
 
-    screen_line(wp, wp->w_winrow, wp->w_wincol, wp->w_width, wp->w_width, -1,
-									    0);
+    screen_line(wp, wp->w_winrow, wp->w_wincol, wp->w_width,
+							  wp->w_width, -1, 0);
 }
 #endif
 
-#if defined(FEAT_FOLDING) || defined(PROTO)
+#if defined(FEAT_FOLDING)
 /*
  * Copy "buf[len]" to ScreenLines["off"] and set attributes to "attr".
  */
@@ -1095,8 +1125,8 @@ fold_line(
 	++col;
     }
 
-#ifdef FEAT_RIGHTLEFT
-# define RL_MEMSET(p, v, l) \
+# ifdef FEAT_RIGHTLEFT
+#  define RL_MEMSET(p, v, l) \
     do { \
 	if (wp->w_p_rl) \
 	    for (ri = 0; ri < (l); ++ri) \
@@ -1105,13 +1135,13 @@ fold_line(
 	    for (ri = 0; ri < (l); ++ri) \
 	       ScreenAttrs[off + (p) + ri] = v; \
     } while (0)
-#else
-# define RL_MEMSET(p, v, l) \
+# else
+#  define RL_MEMSET(p, v, l) \
     do { \
 	for (ri = 0; ri < l; ++ri) \
 	    ScreenAttrs[off + (p) + ri] = v; \
     } while (0)
-#endif
+# endif
 
     // 2. Add the 'foldcolumn'
     //    Reduce the width when there is not enough space.
@@ -1132,11 +1162,11 @@ fold_line(
 		ch = mb_ptr2char_adv(&p);
 	    else
 		ch = *p++;
-#ifdef FEAT_RIGHTLEFT
+# ifdef FEAT_RIGHTLEFT
 	    if (wp->w_p_rl)
 		idx = off + wp->w_width - i - 1 - col;
 	    else
-#endif
+# endif
 		idx = off + col + i;
 	    if (enc_utf8)
 	    {
@@ -1164,7 +1194,7 @@ fold_line(
     // text
     RL_MEMSET(col, HL_ATTR(HLF_FL), wp->w_width - col);
 
-#ifdef FEAT_SIGNS
+# ifdef FEAT_SIGNS
     // If signs are being displayed, add two spaces.
     if (signcolumn_on(wp))
     {
@@ -1173,18 +1203,18 @@ fold_line(
 	{
 	    if (len > 2)
 		len = 2;
-# ifdef FEAT_RIGHTLEFT
+#  ifdef FEAT_RIGHTLEFT
 	    if (wp->w_p_rl)
 		// the line number isn't reversed
 		copy_text_attr(off + wp->w_width - len - col,
 					(char_u *)"  ", len, HL_ATTR(HLF_FL));
 	    else
-# endif
+#  endif
 		copy_text_attr(off + col, (char_u *)"  ", len, HL_ATTR(HLF_FL));
 	    col += len;
 	}
     }
-#endif
+# endif
 
     // 3. Add the 'number' or 'relativenumber' column
     if (wp->w_p_nu || wp->w_p_rnu)
@@ -1216,13 +1246,13 @@ fold_line(
 	    }
 
 	    vim_snprintf((char *)buf, sizeof(buf), fmt, w, num);
-#ifdef FEAT_RIGHTLEFT
+# ifdef FEAT_RIGHTLEFT
 	    if (wp->w_p_rl)
 		// the line number isn't reversed
 		copy_text_attr(off + wp->w_width - len - col, buf, len,
 							     HL_ATTR(HLF_FL));
 	    else
-#endif
+# endif
 		copy_text_attr(off + col, buf, len, HL_ATTR(HLF_FL));
 	    col += len;
 	}
@@ -1240,35 +1270,33 @@ fold_line(
     col = text_to_screenline(wp, text, col);
 
     // Fill the rest of the line with the fold filler
-#ifdef FEAT_RIGHTLEFT
-    if (wp->w_p_rl)
-	col -= txtcol;
-#endif
-    while (col < wp->w_width
-#ifdef FEAT_RIGHTLEFT
-		    - (wp->w_p_rl ? txtcol : 0)
-#endif
-	    )
+    while (col < wp->w_width)
     {
 	int c = wp->w_fill_chars.fold;
+	int idx = off + col;
+
+# ifdef FEAT_RIGHTLEFT
+	if (wp->w_p_rl)
+	    idx = off + wp->w_width - 1 - col;
+# endif
 
 	if (enc_utf8)
 	{
 	    if (c >= 0x80)
 	    {
-		ScreenLinesUC[off + col] = c;
-		ScreenLinesC[0][off + col] = 0;
-		ScreenLines[off + col] = 0x80; // avoid storing zero
+		ScreenLinesUC[idx] = c;
+		ScreenLinesC[0][idx] = 0;
+		ScreenLines[idx] = 0x80; // avoid storing zero
 	    }
 	    else
 	    {
-		ScreenLinesUC[off + col] = 0;
-		ScreenLines[off + col] = c;
+		ScreenLinesUC[idx] = 0;
+		ScreenLines[idx] = c;
 	    }
-	    col++;
 	}
 	else
-	    ScreenLines[off + col++] = c;
+	    ScreenLines[idx] = c;
+	++col;
     }
 
     if (text != buf)
@@ -1324,7 +1352,7 @@ fold_line(
 	}
     }
 
-#ifdef FEAT_SYN_HL
+# ifdef FEAT_SYN_HL
     // Show colorcolumn in the fold line, but let cursorcolumn override it.
     if (wp->w_p_cc_cols)
     {
@@ -1359,10 +1387,10 @@ fold_line(
 	    ScreenAttrs[off + txtcol] = hl_combine_attr(
 				 ScreenAttrs[off + txtcol], HL_ATTR(HLF_CUC));
     }
-#endif
+# endif
 
-    screen_line(wp, row + W_WINROW(wp), wp->w_wincol,
-					      wp->w_width, wp->w_width, -1, 0);
+    screen_line(wp, row + W_WINROW(wp), wp->w_wincol, wp->w_width, wp->w_width,
+	    -1, 0);
 
     // Update w_cline_height and w_cline_folded if the cursor line was
     // updated (saves a call to plines() later).
@@ -1411,7 +1439,7 @@ fold_line(
  *		   - continue redrawing when syntax status is invalid.
  *		4. if scrolled up, update lines at the bottom.
  * This results in three areas that may need updating:
- * top:	from first row to top_end (when scrolled down)
+ * top: from first row to top_end (when scrolled down)
  * mid: from mid_start to mid_end (update inversion or changed text)
  * bot: from bot_start to last row (when scrolled up)
  */
@@ -1430,6 +1458,8 @@ win_update(win_T *wp)
 				// updating.  999 when no bot area updating
     int		scrolled_down = FALSE;	// TRUE when scrolled down when
 					// w_topline got smaller a bit
+    int		scrolled_for_mod = FALSE;   // TRUE after scrolling for changed
+					    // lines
 #ifdef FEAT_SEARCH_EXTRA
     int		top_to_mod = FALSE;    // redraw above mod_top
 #endif
@@ -1904,13 +1934,8 @@ win_update(win_T *wp)
 		    // Correct the first entry for filler lines at the top
 		    // when it won't get updated below.
 		    if (wp->w_p_diff && bot_start > 0)
-		    {
-			int n = plines_win_nofill(wp, wp->w_topline, FALSE)
-			      + wp->w_topfill - adjust_plines_for_skipcol(wp);
-			if (n > wp->w_height)
-			    n = wp->w_height;
-			wp->w_lines[0].wl_size = n;
-		    }
+			wp->w_lines[0].wl_size = plines_correct_topline(wp,
+							  wp->w_topline, TRUE);
 #endif
 		}
 	    }
@@ -2121,11 +2146,11 @@ win_update(win_T *wp)
 		else if (!scrolled_down)
 		    srow += wp->w_lines[idx].wl_size;
 		++idx;
-# ifdef FEAT_FOLDING
+#ifdef FEAT_FOLDING
 		if (idx < wp->w_lines_valid && wp->w_lines[idx].wl_valid)
 		    lnum = wp->w_lines[idx].wl_lnum;
 		else
-# endif
+#endif
 		    ++lnum;
 	    }
 	    srow += mid_start;
@@ -2270,13 +2295,15 @@ win_update(win_T *wp)
 	    // When at start of changed lines: May scroll following lines
 	    // up or down to minimize redrawing.
 	    // Don't do this when the change continues until the end.
-	    // Don't scroll when dollar_vcol >= 0, keep the "$".
-	    // Don't scroll when redrawing the top, scrolled already above.
-	    if (lnum == mod_top
-		    && mod_bot != MAXLNUM
-		    && !(dollar_vcol >= 0 && mod_bot == mod_top + 1)
-		    && row >= top_end)
+	    // Don't scroll for changed lines in the top area if that's already
+	    // done above, but do scroll for changed lines below the top area.
+	    if (!scrolled_for_mod && mod_bot != MAXLNUM
+		    && lnum >= mod_top && lnum < MAX(mod_bot, mod_top + 1)
+		    && (!scrolled_down || row >= top_end))
 	    {
+		scrolled_for_mod = TRUE;
+
+		int		old_cline_height = 0;
 		int		old_rows = 0;
 		int		new_rows = 0;
 		int		xtra_rows;
@@ -2292,6 +2319,8 @@ win_update(win_T *wp)
 		    if (wp->w_lines[i].wl_valid
 			    && wp->w_lines[i].wl_lnum == mod_bot)
 			break;
+		    if (wp->w_lines[i].wl_lnum == wp->w_cursor.lnum)
+			old_cline_height = wp->w_lines[i].wl_size;
 		    old_rows += wp->w_lines[i].wl_size;
 #ifdef FEAT_FOLDING
 		    if (wp->w_lines[i].wl_valid
@@ -2322,26 +2351,17 @@ win_update(win_T *wp)
 		    j = idx;
 		    for (l = lnum; l < mod_bot; ++l)
 		    {
+			if (dollar_vcol >= 0 && wp == curwin &&
+				old_cline_height > 0 && l == wp->w_cursor.lnum)
+			    // When dollar_vcol >= 0, cursor line isn't fully
+			    // redrawn, and its height remains unchanged.
+			    new_rows += old_cline_height;
 #ifdef FEAT_FOLDING
-			if (hasFoldingWin(wp, l, NULL, &l, TRUE, NULL))
+			else if (hasFoldingWin(wp, l, NULL, &l, TRUE, NULL))
 			    ++new_rows;
+#endif
 			else
-#endif
-			{
-#ifdef FEAT_DIFF
-			    if (l == wp->w_topline)
-			    {
-				int n = plines_win_nofill(wp, l, FALSE)
-								+ wp->w_topfill;
-				n -= adjust_plines_for_skipcol(wp);
-				if (n > wp->w_height)
-				    n = wp->w_height;
-				new_rows += n;
-			    }
-			    else
-#endif
-				new_rows += plines_win(wp, l, TRUE);
-			}
+			    new_rows += plines_correct_topline(wp, l, TRUE);
 			++j;
 			if (new_rows > wp->w_height - row - 2)
 			{
@@ -2508,18 +2528,20 @@ win_update(win_T *wp)
 	    wp->w_lines[idx].wl_lnum = lnum;
 	    wp->w_lines[idx].wl_valid = TRUE;
 
+	    int is_curline = wp == curwin && lnum == wp->w_cursor.lnum;
+
 	    // Past end of the window or end of the screen. Note that after
 	    // resizing wp->w_height may be end up too big. That's a problem
 	    // elsewhere, but prevent a crash here.
 	    if (row > wp->w_height || row + wp->w_winrow >= Rows)
 	    {
 		// we may need the size of that too long line later on
-		if (dollar_vcol == -1)
+		if (dollar_vcol == -1 || !is_curline)
 		    wp->w_lines[idx].wl_size = plines_win(wp, lnum, TRUE);
 		++idx;
 		break;
 	    }
-	    if (dollar_vcol == -1)
+	    if (dollar_vcol == -1 || !is_curline)
 		wp->w_lines[idx].wl_size = row - srow;
 	    ++idx;
 #ifdef FEAT_FOLDING
@@ -2596,13 +2618,16 @@ win_update(win_T *wp)
 
 	for (k = 0; k < Rows; ++k)
 	    if (enc_utf8)
-		if ((*mb_off2cells)(LineOffset[k] + Columns - 2,
+		if ((*mb_off2cells)(LineOffset[k] + topframe->fr_width - 2,
 					   LineOffset[k] + screen_Columns) > 1)
-		    screen_draw_rectangle(k, Columns - 2, 1, 2, FALSE);
+		    screen_draw_rectangle(k, topframe->fr_width - 2, 1, 2,
+			    FALSE);
 		else
-		    screen_draw_rectangle(k, Columns - 1, 1, 1, FALSE);
+		    screen_draw_rectangle(k, topframe->fr_width - 1, 1, 1,
+			    FALSE);
 	    else
-		screen_char(LineOffset[k] + Columns - 1, k, Columns - 1);
+		screen_char(LineOffset[k] + topframe->fr_width - 1, k,
+			cmdline_width - 1);
     }
 #endif
 
@@ -2672,7 +2697,7 @@ win_update(win_T *wp)
 	    // Last line isn't finished: Display "@@@" at the end.
 	    screen_fill(W_WINROW(wp) + wp->w_height - 1,
 		    W_WINROW(wp) + wp->w_height,
-		    start_col < wp->w_wincol ? wp->w_wincol : start_col,
+		    (start_col < wp->w_wincol ? wp->w_wincol : start_col),
 		    (int)W_ENDCOL(wp),
 		    symbol, symbol, HL_ATTR(HLF_AT));
 	    set_empty_rows(wp, srow);
@@ -2707,7 +2732,7 @@ win_update(win_T *wp)
 	    }
 #endif
 	}
-	else if (dollar_vcol == -1)
+	else if (dollar_vcol == -1 || wp != curwin)
 	    wp->w_botline = lnum;
 
 	// Make sure the rest of the screen is blank.
@@ -2732,7 +2757,7 @@ win_update(win_T *wp)
     wp->w_old_botfill = wp->w_botfill;
 #endif
 
-    if (dollar_vcol == -1)
+    if (dollar_vcol == -1 || wp != curwin)
     {
 	// There is a trick with w_botline.  If we invalidate it on each
 	// change that might modify it, this will cause a lot of expensive
@@ -2815,19 +2840,19 @@ update_prepare(void)
 {
     cursor_off();
     updating_screen = TRUE;
-#ifdef FEAT_GUI
+# ifdef FEAT_GUI
     // Remove the cursor before starting to do anything, because scrolling may
     // make it difficult to redraw the text under it.
     if (gui.in_use)
 	gui_undraw_cursor();
-#endif
-#ifdef FEAT_SEARCH_EXTRA
+# endif
+# ifdef FEAT_SEARCH_EXTRA
     start_search_hl();
-#endif
-#ifdef FEAT_PROP_POPUP
+# endif
+# ifdef FEAT_PROP_POPUP
     // Update popup_mask if needed.
     may_update_popup_mask(must_redraw);
-#endif
+# endif
 }
 
 /*
@@ -2857,7 +2882,7 @@ update_finish(void)
 }
 #endif
 
-#if defined(FEAT_NETBEANS_INTG) || defined(PROTO)
+#if defined(FEAT_NETBEANS_INTG)
     void
 update_debug_sign(buf_T *buf, linenr_T lnum)
 {
@@ -2881,9 +2906,9 @@ update_debug_sign(buf_T *buf, linenr_T lnum)
     if (!doit || updating_screen
 	    || State == MODE_ASKMORE || State == MODE_HITRETURN
 	    || msg_scrolled
-#ifdef FEAT_GUI
+# ifdef FEAT_GUI
 	    || gui.starting
-#endif
+# endif
 	    || starting)
 	return;
 
@@ -2898,11 +2923,16 @@ update_debug_sign(buf_T *buf, linenr_T lnum)
 	    win_redr_status(wp, FALSE);
     }
 
+# if defined(FEAT_TABPANEL)
+    if (redraw_tabpanel)
+	draw_tabpanel();
+# endif
+
     update_finish();
 }
 #endif
 
-#if defined(FEAT_GUI) || defined(PROTO)
+#if defined(FEAT_GUI)
 /*
  * Update a single window, its status line and maybe the command line msg.
  * Used for the GUI scrollbar.
@@ -2916,13 +2946,13 @@ updateWindow(win_T *wp)
 
     update_prepare();
 
-#ifdef FEAT_CLIPBOARD
+# ifdef FEAT_CLIPBOARD
     // When Visual area changed, may have to update selection.
     if (clip_star.available && clip_isautosel_star())
 	clip_update_selection(&clip_star);
     if (clip_plus.available && clip_isautosel_plus())
 	clip_update_selection(&clip_plus);
-#endif
+# endif
 
     win_update(wp);
 
@@ -2930,6 +2960,11 @@ updateWindow(win_T *wp)
     if (redraw_tabline)
 	draw_tabline();
 
+# if defined(FEAT_TABPANEL)
+    if (redraw_tabpanel)
+	draw_tabpanel();
+# endif
+
     if (wp->w_redr_status || p_ru
 # ifdef FEAT_STL_OPT
 	    || *p_stl != NUL || *wp->w_p_stl != NUL
@@ -2937,10 +2972,10 @@ updateWindow(win_T *wp)
 	    )
 	win_redr_status(wp, FALSE);
 
-#ifdef FEAT_PROP_POPUP
+# ifdef FEAT_PROP_POPUP
     // Display popup windows on top of everything.
     update_popups(win_update);
-#endif
+# endif
 
     update_finish();
 }
@@ -3163,6 +3198,13 @@ redraw_win_later(
     win_T	*wp,
     int		type)
 {
+#ifdef FEAT_EVAL
+    // If inside a redraw_listener_add() callback, then only set the redraw type
+    // for the window and not request another one right after.
+    if (inside_redraw_on_start_cb && wp->w_redr_type < type)
+	wp->w_redr_type = type;
+    else
+#endif
     if (!exiting && !redraw_not_allowed && wp->w_redr_type < type)
     {
 	wp->w_redr_type = type;
@@ -3206,9 +3248,9 @@ redraw_all_later(int type)
 redraw_all_windows_later(int type)
 {
     redraw_all_later(type);
-#ifdef FEAT_PROP_POPUP
+# ifdef FEAT_PROP_POPUP
     popup_redraw_all();		// redraw all popup windows
-#endif
+# endif
 }
 #endif
 
@@ -3219,7 +3261,11 @@ redraw_all_windows_later(int type)
     void
 set_must_redraw(int type)
 {
-    if (!redraw_not_allowed && must_redraw < type)
+    if (!redraw_not_allowed &&
+#ifdef FEAT_EVAL
+	    !inside_redraw_on_start_cb &&
+#endif
+	    must_redraw < type)
 	must_redraw = type;
 }
 
@@ -3249,7 +3295,7 @@ redraw_buf_later(buf_T *buf, int type)
 #endif
 }
 
-#if defined(FEAT_SIGNS) || defined(PROTO)
+#if defined(FEAT_SIGNS)
     void
 redraw_buf_line_later(buf_T *buf, linenr_T lnum)
 {
@@ -3262,7 +3308,7 @@ redraw_buf_line_later(buf_T *buf, linenr_T lnum)
 }
 #endif
 
-#if defined(FEAT_JOB_CHANNEL) || defined(PROTO)
+#if defined(FEAT_JOB_CHANNEL)
     void
 redraw_buf_and_status_later(buf_T *buf, int type)
 {
@@ -3328,6 +3374,11 @@ redraw_statuslines(void)
 	    win_redr_status(wp, FALSE);
     if (redraw_tabline)
 	draw_tabline();
+
+#if defined(FEAT_TABPANEL)
+    if (redraw_tabpanel)
+	draw_tabpanel();
+#endif
 }
 
 /*
@@ -3383,3 +3434,187 @@ redraw_win_range_later(
 	redraw_win_later(wp, UPD_VALID);
     }
 }
+
+#ifdef FEAT_EVAL
+static bool redraw_cb_in_progress = false;
+
+    void
+f_redraw_listener_add(typval_T *argvars, typval_T *rettv)
+{
+    redraw_listener_T	*rln;
+    dict_T		*dict;
+    typval_T		tv;
+    bool		got_one = false;
+    static int		id;
+
+    if (redraw_cb_in_progress)
+    {
+	emsg(_(e_cannot_add_redraw_listener_in_listener_callback));
+	return;
+    }
+
+    if (check_for_dict_arg(argvars, 0) == FAIL)
+	return;
+
+    rln = ALLOC_CLEAR_ONE(redraw_listener_T);
+
+    if (rln == NULL)
+	return;
+    dict = argvars[0].vval.v_dict;
+
+    /*
+     * on_start: called on each screen redraw
+     *
+     * on_end: called at the end of a redraw cycle
+     */
+    if (dict_get_tv(dict, "on_start", &tv) == OK)
+    {
+	callback_T cb = get_callback(&tv);
+
+	if (cb.cb_name == NULL)
+	{
+	    clear_tv(&tv);
+	    vim_free(rln);
+	    return;
+	}
+	copy_callback(&rln->rl_callbacks.on_start, &cb);
+	free_callback(&cb);
+	clear_tv(&tv);
+	got_one = true;
+    }
+
+    if (dict_get_tv(dict, "on_end", &tv) == OK)
+    {
+	callback_T cb = get_callback(&tv);
+
+	if (cb.cb_name == NULL)
+	{
+	    clear_tv(&tv);
+	    free_callback(&rln->rl_callbacks.on_start);
+	    vim_free(rln);
+	    return;
+	}
+	copy_callback(&rln->rl_callbacks.on_end, &cb);
+	free_callback(&cb);
+	clear_tv(&tv);
+	got_one = true;
+    }
+
+    if (!got_one)
+    {
+	emsg(_(e_no_redraw_listener_callbacks_defined));
+	vim_free(rln);
+	return;
+    }
+
+    rln->rl_next = redraw_listeners;
+    redraw_listeners = rln;
+    rln->rl_id = ++id; // Never zero
+
+    rettv->v_type = VAR_NUMBER;
+    rettv->vval.v_number = id;
+
+    return;
+}
+
+    static void
+redraw_listener_free(redraw_listener_T *rln)
+{
+    free_callback(&rln->rl_callbacks.on_start);
+    free_callback(&rln->rl_callbacks.on_end);
+
+    vim_free(rln);
+}
+
+
+    static void
+redraw_listener_cleanup(void)
+{
+    for (redraw_listener_T *rln = redraw_listeners; rln != NULL;)
+    {
+	redraw_listener_T *next = rln->rl_next;
+	if (rln->rl_id == 0)
+	{
+	    if (redraw_listeners == rln)
+		redraw_listeners = rln->rl_next;
+	    redraw_listener_free(rln);
+	}
+	rln = next;
+    }
+}
+
+/*
+ * Return the redraw listener struct with the specified id. Returns NULL if not
+ * found.
+ */
+    static redraw_listener_T *
+get_redraw_listener(int id)
+{
+    for (redraw_listener_T *rln = redraw_listeners; rln != NULL; rln = rln->rl_next)
+	if (rln->rl_id == id)
+	    return rln;
+    return NULL;
+}
+
+    void
+f_redraw_listener_remove(typval_T *argvars, typval_T *rettv UNUSED)
+{
+    int			id;
+    redraw_listener_T	*rln;
+
+    if (check_for_number_arg(argvars, 0) == FAIL)
+	return;
+
+    id = argvars[0].vval.v_number;
+    rln = get_redraw_listener(id);
+
+    rettv->v_type = VAR_NUMBER;
+    if (rln == NULL)
+    {
+	rettv->vval.v_number = 0;
+	return;
+    }
+
+    // We set the id to zero instead of freeing it here, since we still need
+    // rl_next from it.
+    rln->rl_id = 0;
+    rettv->vval.v_number = 1;
+}
+
+/*
+ * Invoke the on_start callbacks.
+ */
+    static void
+invoke_redraw_listener_start_or_end(bool start)
+{
+    typval_T argv[1];
+    typval_T rettv;
+
+    argv[0].v_type = VAR_UNKNOWN;
+
+    if (start)
+	inside_redraw_on_start_cb = true;
+
+    redraw_cb_in_progress = true;
+    for (redraw_listener_T *rln = redraw_listeners; rln != NULL; rln = rln->rl_next)
+    {
+	if (rln->rl_id == 0)
+	    // Listener has been removed, skip
+	    continue;
+	if (start && rln->rl_callbacks.on_start.cb_name != NULL)
+	{
+	    call_callback(&rln->rl_callbacks.on_start, -1, &rettv, 0, argv);
+	    clear_tv(&rettv);
+	}
+	else if (rln->rl_callbacks.on_end.cb_name != NULL)
+	{
+	    call_callback(&rln->rl_callbacks.on_end, -1, &rettv, 0, argv);
+	    clear_tv(&rettv);
+	}
+    }
+    redraw_cb_in_progress = false;
+
+    if (start)
+	inside_redraw_on_start_cb = false;
+}
+#endif // FEAT_EVAL
